@@ -8,6 +8,29 @@ import os
 import unittest
 
 from PIL import Image, ImageMath
+from PIL._util import py3
+
+import logging
+logger = logging.getLogger(__name__)
+
+
+HAS_UPLOADER = False
+
+if os.environ.get('SHOW_ERRORS', None):
+    # local img.show for errors.
+    HAS_UPLOADER = True
+
+    class test_image_results:
+        @classmethod
+        def upload(self, a, b):
+            a.show()
+            b.show()
+else:
+    try:
+        import test_image_results
+        HAS_UPLOADER = True
+    except ImportError:
+        pass
 
 
 def convert_to_comparable(a, b):
@@ -41,9 +64,8 @@ class PillowTestCase(unittest.TestCase):
     def delete_tempfile(self, path):
         try:
             ok = self.currentResult.wasSuccessful()
-        except AttributeError:  # for nosetests
-            proxy = self.currentResult
-            ok = (len(proxy.errors) + len(proxy.failures) == 0)
+        except AttributeError:  # for pytest
+            ok = True
 
         if ok:
             # only clean out tempfiles if test passed
@@ -84,8 +106,21 @@ class PillowTestCase(unittest.TestCase):
             a.size, b.size,
             msg or "got size %r, expected %r" % (a.size, b.size))
         if a.tobytes() != b.tobytes():
+            if HAS_UPLOADER:
+                try:
+                    url = test_image_results.upload(a, b)
+                    logger.error("Url for test images: %s" % url)
+                except Exception as msg:
+                    pass
+
             self.fail(msg or "got different content")
 
+    def assert_image_equal_tofile(self, a, filename, msg=None, mode=None):
+        with Image.open(filename) as img:
+            if mode:
+                img = img.convert(mode)
+            self.assert_image_equal(a, img, msg)
+
     def assert_image_similar(self, a, b, epsilon, msg=None):
         epsilon = float(epsilon)
         self.assertEqual(
@@ -103,11 +138,26 @@ class PillowTestCase(unittest.TestCase):
             diff += sum(i * num for i, num in enumerate(chdiff.histogram()))
 
         ave_diff = float(diff)/(a.size[0]*a.size[1])
-        self.assertGreaterEqual(
-            epsilon, ave_diff,
-            (msg or '') +
-            " average pixel value difference %.4f > epsilon %.4f" % (
-                ave_diff, epsilon))
+        try:
+            self.assertGreaterEqual(
+                epsilon, ave_diff,
+                (msg or '') +
+                " average pixel value difference %.4f > epsilon %.4f" % (
+                    ave_diff, epsilon))
+        except Exception as e:
+            if HAS_UPLOADER:
+                try:
+                    url = test_image_results.upload(a, b)
+                    logger.error("Url for test images: %s" % url)
+                except:
+                    pass
+            raise e
+
+    def assert_image_similar_tofile(self, a, filename, epsilon, msg=None, mode=None):
+        with Image.open(filename) as img:
+            if mode:
+                img = img.convert(mode)
+            self.assert_image_similar(a, img, epsilon, msg)
 
     def assert_warning(self, warn_class, func, *args, **kwargs):
         import warnings
@@ -135,11 +185,17 @@ class PillowTestCase(unittest.TestCase):
                 self.assertTrue(found)
         return result
 
+    def assert_all_same(self, items, msg=None):
+        self.assertTrue(items.count(items[0]) == len(items), msg)
+
+    def assert_not_all_same(self, items, msg=None):
+        self.assertFalse(items.count(items[0]) == len(items), msg)
+
     def skipKnownBadTest(self, msg=None, platform=None,
                          travis=None, interpreter=None):
         # Skip if platform/travis matches, and
         # PILLOW_RUN_KNOWN_BAD is not true in the environment.
-        if bool(os.environ.get('PILLOW_RUN_KNOWN_BAD', False)):
+        if os.environ.get('PILLOW_RUN_KNOWN_BAD', False):
             print(os.environ.get('PILLOW_RUN_KNOWN_BAD', False))
             return
 
@@ -154,10 +210,6 @@ class PillowTestCase(unittest.TestCase):
         if skip:
             self.skipTest(msg or "Known Bad Test")
 
-    def shortDescription(self):
-        # Prevents `nose -v` printing docstrings
-        return None
-
     def tempfile(self, template):
         assert template[:5] in ("temp.", "temp_")
         fd, path = tempfile.mkstemp(template[4:], template[:4])
@@ -175,32 +227,33 @@ class PillowTestCase(unittest.TestCase):
             return Image.open(outfile)
         raise IOError()
 
-@unittest.skipIf(sys.platform.startswith('win32'), "requires Unix or MacOS")
+
+@unittest.skipIf(sys.platform.startswith('win32'), "requires Unix or macOS")
 class PillowLeakTestCase(PillowTestCase):
-    # requires unix/osx
-    iterations = 100 # count
-    mem_limit = 512 # k
-    
+    # requires unix/macOS
+    iterations = 100  # count
+    mem_limit = 512  # k
+
     def _get_mem_usage(self):
         """
         Gets the RUSAGE memory usage, returns in K. Encapsulates the difference
-        between OSX and Linux rss reporting
+        between macOS and Linux rss reporting
 
-        :returns; memory usage in kilobytes
+        :returns: memory usage in kilobytes
         """
-        
-        from resource import getpagesize, getrusage, RUSAGE_SELF
+
+        from resource import getrusage, RUSAGE_SELF
         mem = getrusage(RUSAGE_SELF).ru_maxrss
         if sys.platform == 'darwin':
             # man 2 getrusage:
             #     ru_maxrss    the maximum resident set size utilized (in bytes).
-            return mem / 1024 # Kb
+            return mem / 1024  # Kb
         else:
             # linux
             # man 2 getrusage
             #        ru_maxrss (since Linux 2.6.32)
-            #  This is the maximum resident set size used (in  kilobytes). 
-            return mem # Kb
+            #  This is the maximum resident set size used (in kilobytes).
+            return mem  # Kb
 
     def _test_leak(self, core):
         start_mem = self._get_mem_usage()
@@ -213,7 +266,10 @@ class PillowLeakTestCase(PillowTestCase):
 
 # helpers
 
-py3 = (sys.version_info >= (3, 0))
+if not py3:
+    # Remove DeprecationWarning in Python 3
+    PillowTestCase.assertRaisesRegex = PillowTestCase.assertRaisesRegexp
+    PillowTestCase.assertRegex = PillowTestCase.assertRegexpMatches
 
 
 def fromstring(data):
@@ -283,6 +339,7 @@ def imagemagick_available():
 def on_appveyor():
     return 'APPVEYOR' in os.environ
 
+
 if sys.platform == 'win32':
     IMCONVERT = os.environ.get('MAGICK_HOME', '')
     if IMCONVERT:
