@@ -1,5 +1,6 @@
 from helper import unittest, PillowTestCase, PillowLeakTestCase, hopper
 from PIL import Image, ImageFile, PngImagePlugin
+from PIL._util import py3
 
 from io import BytesIO
 import zlib
@@ -22,6 +23,7 @@ def chunk(cid, *data):
     PngImagePlugin.putchunk(*(test_file, cid) + data)
     return test_file.getvalue()
 
+
 o32 = PngImagePlugin.o32
 
 IHDR = chunk(b"IHDR", o32(1), o32(1), b'\x08\x02', b'\0\0\0')
@@ -49,11 +51,25 @@ class TestFilePng(PillowTestCase):
         if "zip_encoder" not in codecs or "zip_decoder" not in codecs:
             self.skipTest("zip/deflate support not available")
 
+    def get_chunks(self, filename):
+        chunks = []
+        with open(filename, "rb") as fp:
+            fp.read(8)
+            with PngImagePlugin.PngStream(fp) as png:
+                while True:
+                    cid, pos, length = png.read()
+                    chunks.append(cid)
+                    try:
+                        s = png.call(cid, pos, length)
+                    except EOFError:
+                        break
+                    png.crc(cid, s)
+        return chunks
+
     def test_sanity(self):
 
         # internal version number
-        self.assertRegexpMatches(
-            Image.core.zlib_version, r"\d+\.\d+\.\d+(\.\d+)?$")
+        self.assertRegex(Image.core.zlib_version, r"\d+\.\d+\.\d+(\.\d+)?$")
 
         test_file = self.tempfile("temp.png")
 
@@ -276,15 +292,29 @@ class TestFilePng(PillowTestCase):
         self.assertEqual(im.getcolors(), [(100, (0, 0, 0, 0))])
 
     def test_save_l_transparency(self):
+        # There are 559 transparent pixels in l_trns.png.
+        num_transparent = 559
+
         in_file = "Tests/images/l_trns.png"
         im = Image.open(in_file)
+        self.assertEqual(im.mode, "L")
+        self.assertEqual(im.info["transparency"], 255)
+
+        im_rgba = im.convert('RGBA')
+        self.assertEqual(
+            im_rgba.getchannel("A").getcolors()[0][0], num_transparent)
 
         test_file = self.tempfile("temp.png")
         im.save(test_file)
 
-        # There are 559 transparent pixels.
-        im = im.convert('RGBA')
-        self.assertEqual(im.getchannel('A').getcolors()[0][0], 559)
+        test_im = Image.open(test_file)
+        self.assertEqual(test_im.mode, "L")
+        self.assertEqual(test_im.info["transparency"], 255)
+        self.assert_image_equal(im, test_im)
+
+        test_im_rgba = test_im.convert('RGBA')
+        self.assertEqual(
+            test_im_rgba.getchannel('A').getcolors()[0][0], num_transparent)
 
     def test_save_rgb_single_transparency(self):
         in_file = "Tests/images/caption_6_33_22.png"
@@ -360,7 +390,7 @@ class TestFilePng(PillowTestCase):
 
         info = PngImagePlugin.PngInfo()
         info.add_text("TXT", "VALUE")
-        info.add_text("ZIP", "VALUE", 1)
+        info.add_text("ZIP", "VALUE", zip=True)
 
         im = roundtrip(im, pnginfo=info)
         self.assertEqual(im.info, {'TXT': 'VALUE', 'ZIP': 'VALUE'})
@@ -404,7 +434,7 @@ class TestFilePng(PillowTestCase):
             im = roundtrip(im, pnginfo=info)
             self.assertEqual(im.info, {"Text": value})
 
-        if str is not bytes:
+        if py3:
             rt_text(" Aa" + chr(0xa0) + chr(0xc4) + chr(0xff))  # Latin1
             rt_text(chr(0x400) + chr(0x472) + chr(0x4ff))       # Cyrillic
             rt_text(chr(0x4e00) + chr(0x66f0) +                 # CJK
@@ -501,18 +531,7 @@ class TestFilePng(PillowTestCase):
         test_file = self.tempfile("temp.png")
         im.convert("P").save(test_file, dpi=(100, 100))
 
-        chunks = []
-        with open(test_file, "rb") as fp:
-            fp.read(8)
-            png = PngImagePlugin.PngStream(fp)
-            while True:
-                cid, pos, length = png.read()
-                chunks.append(cid)
-                try:
-                    s = png.call(cid, pos, length)
-                except EOFError:
-                    break
-                png.crc(cid, s)
+        chunks = self.get_chunks(test_file)
 
         # https://www.w3.org/TR/PNG/#5ChunkOrdering
         # IHDR - shall be first
@@ -535,10 +554,10 @@ class TestFilePng(PillowTestCase):
         self.assertEqual(len(chunks), 3)
 
 
-@unittest.skipIf(sys.platform.startswith('win32'), "requires Unix or MacOS")
+@unittest.skipIf(sys.platform.startswith('win32'), "requires Unix or macOS")
 class TestTruncatedPngPLeaks(PillowLeakTestCase):
     mem_limit = 2*1024  # max increase in K
-    iterations = 100 # Leak is 56k/iteration, this will leak 5.6megs
+    iterations = 100  # Leak is 56k/iteration, this will leak 5.6megs
 
     def setUp(self):
         if "zip_encoder" not in codecs or "zip_decoder" not in codecs:
