@@ -3,9 +3,10 @@ from io import BytesIO
 import struct
 import sys
 
-from helper import unittest, PillowTestCase, hopper, py3
+from helper import unittest, PillowTestCase, hopper
 
 from PIL import Image, TiffImagePlugin
+from PIL._util import py3
 from PIL.TiffImagePlugin import X_RESOLUTION, Y_RESOLUTION, RESOLUTION_UNIT
 
 logger = logging.getLogger(__name__)
@@ -51,43 +52,22 @@ class TestFileTiff(PillowTestCase):
         self.assertEqual(im.tile, [('raw', (0, 0, 55, 43), 8, ('RGBa', 0, 1))])
         im.load()
 
-    def test_16bit_RGBa_tiff(self):
-        im = Image.open("Tests/images/tiff_16bit_RGBa.tiff")
+        self.assert_image_similar_tofile(im, "Tests/images/pil136.png", 1)
+
+    def test_wrong_bits_per_sample(self):
+        im = Image.open("Tests/images/tiff_wrong_bits_per_sample.tiff")
 
         self.assertEqual(im.mode, "RGBA")
-        self.assertEqual(im.size, (100, 40))
-        self.assertEqual(im.tile, [('tiff_lzw', (0, 0, 100, 40), 50, 'RGBa;16B')])
+        self.assertEqual(im.size, (52, 53))
+        self.assertEqual(im.tile, [('raw', (0, 0, 52, 53), 160, ('RGBA', 0, 1))])
         im.load()
 
-    def test_gimp_tiff(self):
-        # Read TIFF JPEG images from GIMP [@PIL168]
-
-        codecs = dir(Image.core)
-        if "jpeg_decoder" not in codecs:
-            self.skipTest("jpeg support not available")
-
-        filename = "Tests/images/pil168.tif"
-        im = Image.open(filename)
-
-        self.assertEqual(im.mode, "RGB")
-        self.assertEqual(im.size, (256, 256))
-        self.assertEqual(
-            im.tile, [
-                ('jpeg', (0, 0, 256, 64), 8, ('RGB', '')),
-                ('jpeg', (0, 64, 256, 128), 1215, ('RGB', '')),
-                ('jpeg', (0, 128, 256, 192), 2550, ('RGB', '')),
-                ('jpeg', (0, 192, 256, 256), 3890, ('RGB', '')),
-                ])
-        im.load()
-
-    def test_sampleformat(self):
-        # https://github.com/python-pillow/Pillow/issues/1466
-        im = Image.open("Tests/images/copyleft.tiff")
-        self.assertEqual(im.mode, 'RGB')
-
     def test_set_legacy_api(self):
-        with self.assertRaises(Exception):
-            ImageFileDirectory_v2.legacy_api = None
+        ifd = TiffImagePlugin.ImageFileDirectory_v2()
+        with self.assertRaises(Exception) as e:
+            ifd.legacy_api = None
+        self.assertEqual(str(e.exception),
+                         "Not allowing setting of legacy api")
 
     def test_xyres_tiff(self):
         filename = "Tests/images/pil168.tif"
@@ -202,8 +182,8 @@ class TestFileTiff(PillowTestCase):
         im = Image.open('Tests/images/16bit.s.tif')
         im.load()
         self.assertEqual(im.mode, 'I')
-        self.assertEqual(im.getpixel((0,0)),32767)
-        self.assertEqual(im.getpixel((0,1)),0)
+        self.assertEqual(im.getpixel((0, 0)), 32767)
+        self.assertEqual(im.getpixel((0, 1)), 0)
 
     def test_12bit_rawmode(self):
         """ Are we generating the same interpretation
@@ -217,12 +197,7 @@ class TestFileTiff(PillowTestCase):
         # imagemagick will auto scale so that a 12bit FFF is 16bit FFF0,
         # so we need to unshift so that the integer values are the same.
 
-        im2 = Image.open('Tests/images/12in16bit.tif')
-
-        logger.debug("%s", [img.getpixel((0, idx))
-                            for img in [im, im2] for idx in range(3)])
-
-        self.assert_image_equal(im, im2)
+        self.assert_image_equal_tofile(im, 'Tests/images/12in16bit.tif')
 
     def test_32bit_float(self):
         # Issue 614, specific 32-bit float format
@@ -417,7 +392,6 @@ class TestFileTiff(PillowTestCase):
                   'y_resolution': 36}
         filename = self.tempfile("temp.tif")
         hopper("RGB").save(filename, **kwargs)
-        from PIL.TiffImagePlugin import X_RESOLUTION, Y_RESOLUTION
         im = Image.open(filename)
 
         # legacy interface
@@ -428,17 +402,6 @@ class TestFileTiff(PillowTestCase):
         self.assertEqual(im.tag_v2[X_RESOLUTION], 72)
         self.assertEqual(im.tag_v2[Y_RESOLUTION], 36)
 
-    def test_lzw(self):
-        # Act
-        im = Image.open("Tests/images/hopper_lzw.tif")
-
-        # Assert
-        self.assertEqual(im.mode, 'RGB')
-        self.assertEqual(im.size, (128, 128))
-        self.assertEqual(im.format, "TIFF")
-        im2 = hopper()
-        self.assert_image_similar(im, im2, 5)
-
     def test_roundtrip_tiff_uint16(self):
         # Test an image of all '0' values
         pixel_value = 0x1234
@@ -470,12 +433,22 @@ class TestFileTiff(PillowTestCase):
         im = Image.new('RGB', (100, 100), '#f00')
         ims = [Image.new('RGB', (100, 100), color) for color
                in ['#0f0', '#00f']]
-        im.save(mp, format="TIFF", save_all=True, append_images=ims)
+        im.copy().save(mp, format="TIFF", save_all=True, append_images=ims)
 
         mp.seek(0, os.SEEK_SET)
         reread = Image.open(mp)
         self.assertEqual(reread.n_frames, 3)
 
+        # Test appending using a generator
+        def imGenerator(ims):
+            for im in ims:
+                yield im
+        mp = io.BytesIO()
+        im.save(mp, format="TIFF", save_all=True, append_images=imGenerator(ims))
+
+        mp.seek(0, os.SEEK_SET)
+        reread = Image.open(mp)
+        self.assertEqual(reread.n_frames, 3)
 
     def test_saving_icc_profile(self):
         # Tests saving TIFF with icc_profile set.
@@ -518,6 +491,7 @@ class TestFileTiff(PillowTestCase):
             im.load()
             self.assertFalse(fp.closed)
 
+
 @unittest.skipUnless(sys.platform.startswith('win32'), "Windows only")
 class TestFileTiffW32(PillowTestCase):
     def test_fd_leak(self):
@@ -531,7 +505,7 @@ class TestFileTiffW32(PillowTestCase):
         im = Image.open(tmpfile)
         fp = im.fp
         self.assertFalse(fp.closed)
-        self.assertRaises(Exception, os.remove, tmpfile)
+        self.assertRaises(WindowsError, os.remove, tmpfile)
         im.load()
         self.assertTrue(fp.closed)
 
