@@ -1,6 +1,7 @@
 from helper import unittest, PillowTestCase, hopper
 
 from PIL import Image
+from PIL._util import py3
 import os
 
 
@@ -22,7 +23,7 @@ class TestImage(PillowTestCase):
             'BGR;15', 'BGR;16', 'BGR;24', 'BGR;32'
         ]:
             with self.assertRaises(ValueError) as e:
-                Image.new(mode, (1, 1));
+                Image.new(mode, (1, 1))
             self.assertEqual(str(e.exception), 'unrecognized image mode')
 
     def test_sanity(self):
@@ -60,12 +61,12 @@ class TestImage(PillowTestCase):
         self.assertEqual(im.height, 4)
 
     def test_invalid_image(self):
-        if str is bytes:
-            import StringIO
-            im = StringIO.StringIO('')
-        else:
+        if py3:
             import io
             im = io.BytesIO(b'')
+        else:
+            import StringIO
+            im = StringIO.StringIO('')
         self.assertRaises(IOError, Image.open, im)
 
     def test_bad_mode(self):
@@ -112,7 +113,6 @@ class TestImage(PillowTestCase):
         self.assertRaises(ValueError, im.save, temp_file)
 
     def test_internals(self):
-
         im = Image.new("L", (100, 100))
         im.readonly = 1
         im._copy()
@@ -122,8 +122,15 @@ class TestImage(PillowTestCase):
         im.paste(0, (0, 0, 100, 100))
         self.assertFalse(im.readonly)
 
-        test_file = self.tempfile("temp.ppm")
-        im._dump(test_file)
+    def test_dump(self):
+        im = Image.new("L", (10, 10))
+        im._dump(self.tempfile("temp_L.ppm"))
+
+        im = Image.new("RGB", (10, 10))
+        im._dump(self.tempfile("temp_RGB.ppm"))
+
+        im = Image.new("HSV", (10, 10))
+        self.assertRaises(ValueError, im._dump, self.tempfile("temp_HSV.ppm"))
 
     def test_comparison_with_other_type(self):
         # Arrange
@@ -236,9 +243,9 @@ class TestImage(PillowTestCase):
         self.assertEqual(img_colors, expected_colors)
 
     def test_alpha_inplace(self):
-        src = Image.new('RGBA', (128,128), 'blue')
+        src = Image.new('RGBA', (128, 128), 'blue')
 
-        over = Image.new('RGBA', (128,128), 'red')
+        over = Image.new('RGBA', (128, 128), 'red')
         mask = hopper('L')
         over.putalpha(mask)
 
@@ -275,17 +282,18 @@ class TestImage(PillowTestCase):
 
         # errors
         self.assertRaises(ValueError,
-            source.alpha_composite, over, "invalid source")
+                          source.alpha_composite, over, "invalid source")
         self.assertRaises(ValueError,
-            source.alpha_composite, over, (0, 0), "invalid destination")
+                          source.alpha_composite, over, (0, 0),
+                          "invalid destination")
         self.assertRaises(ValueError,
-            source.alpha_composite, over, (0))
+                          source.alpha_composite, over, (0))
         self.assertRaises(ValueError,
-            source.alpha_composite, over, (0, 0), (0))
+                          source.alpha_composite, over, (0, 0), (0))
         self.assertRaises(ValueError,
-            source.alpha_composite, over, (0, -1))
+                          source.alpha_composite, over, (0, -1))
         self.assertRaises(ValueError,
-            source.alpha_composite, over, (0, 0), (0, -1))
+                          source.alpha_composite, over, (0, 0), (0, -1))
 
     def test_registered_extensions_uninitialized(self):
         # Arrange
@@ -312,7 +320,7 @@ class TestImage(PillowTestCase):
         extensions = Image.registered_extensions()
 
         # Assert
-        self.assertTrue(bool(extensions))
+        self.assertTrue(extensions)
         for ext in ['.cur', '.icns', '.tif', '.tiff']:
             self.assertIn(ext, extensions)
 
@@ -354,7 +362,12 @@ class TestImage(PillowTestCase):
         # Assert
         self.assertEqual(im.size, (100, 100))
         self.assertEqual(im.mode, "L")
-        self.assertNotEqual(im.getpixel((0, 0)), im.getpixel((0, 1)))
+        p0 = im.getpixel((0, 0))
+        p1 = im.getpixel((0, 1))
+        p2 = im.getpixel((0, 2))
+        p3 = im.getpixel((0, 3))
+        p4 = im.getpixel((0, 4))
+        self.assert_not_all_same([p0, p1, p2, p3, p4])
 
     def test_effect_spread(self):
         # Arrange
@@ -490,8 +503,8 @@ class TestImage(PillowTestCase):
         im = hopper('RGB')
         im_p = hopper('P')
 
-        blank_p = Image.new('P', (10,10))
-        blank_pa = Image.new('PA', (10,10))
+        blank_p = Image.new('P', (10, 10))
+        blank_pa = Image.new('PA', (10, 10))
         blank_p.palette = None
         blank_pa.palette = None
 
@@ -515,10 +528,11 @@ class TestImage(PillowTestCase):
         # https://github.com/python-pillow/Pillow/issues/835
         # Arrange
         test_file = 'Tests/images/hopper.png'
+        temp_file = self.tempfile("temp.jpg")
 
         # Act/Assert
         with Image.open(test_file) as im:
-            self.assert_warning(None, lambda: im.save('test_img.jpg'))
+            self.assert_warning(None, im.save, temp_file)
 
 
 class MockEncoder(object):
