@@ -10,6 +10,7 @@ from PIL import Image
 import sys
 import os
 
+
 class AccessTest(PillowTestCase):
     # initial value
     _init_cffi_access = Image.USE_CFFI_ACCESS
@@ -251,14 +252,13 @@ class TestCffi(AccessTest):
 
 class TestEmbeddable(unittest.TestCase):
     @unittest.skipIf(not sys.platform.startswith('win32') or
-                     sys.version_info[:2] in ((3, 3), (3, 4)) or
+                     sys.version_info[:2] == (3, 4) or
                      on_appveyor(),   # failing on appveyor when run from
                                       # subprocess, not from shell
                      "requires Python 2.7 or >=3.5 for Windows")
     def test_embeddable(self):
         import subprocess
         import ctypes
-        import setuptools
         from distutils import ccompiler, sysconfig
 
         with open('embed_pil.c', 'w') as fh:
@@ -293,22 +293,23 @@ int main(int argc, char* argv[])
 
         compiler = ccompiler.new_compiler()
         compiler.add_include_dir(sysconfig.get_python_inc())
-        
+
         libdir = sysconfig.get_config_var('LIBDIR') or sysconfig.get_python_inc().replace('include', 'libs')
-        print (libdir)
+        print(libdir)
         compiler.add_library_dir(libdir)
         objects = compiler.compile(['embed_pil.c'])
         compiler.link_executable(objects, 'embed_pil')
 
         env = os.environ.copy()
         env["PATH"] = sys.prefix + ';' + env["PATH"]
-        
+
         # do not display the Windows Error Reporting dialog
         ctypes.windll.kernel32.SetErrorMode(0x0002)
-        
+
         process = subprocess.Popen(['embed_pil.exe'], env=env)
         process.communicate()
         self.assertEqual(process.returncode, 0)
 
+
 if __name__ == '__main__':
     unittest.main()
