@@ -74,9 +74,9 @@ class TestImageFile(PillowTestCase):
     def test_ico(self):
         with open('Tests/images/python.ico', 'rb') as f:
             data = f.read()
-        p = ImageFile.Parser()
-        p.feed(data)
-        self.assertEqual((48, 48), p.image.size)
+        with ImageFile.Parser() as p:
+            p.feed(data)
+            self.assertEqual((48, 48), p.image.size)
 
     def test_safeblock(self):
         if "zip_encoder" not in codecs:
@@ -146,6 +146,7 @@ class MockPyDecoder(ImageFile.PyDecoder):
         # eof
         return (-1, 0)
 
+
 xoff, yoff, xsize, ysize = 10, 20, 100, 100
 
 
@@ -222,5 +223,13 @@ class TestPyDecoder(PillowTestCase):
         im.tile = [("MOCK", (xoff, yoff, xoff+xsize, yoff+ysize + 100), 32, None)]
         self.assertRaises(ValueError, im.load)
 
+    def test_no_format(self):
+        buf = BytesIO(b'\x00'*255)
+
+        im = MockImageFile(buf)
+        self.assertIsNone(im.format)
+        self.assertIsNone(im.get_format_mimetype())
+
+
 if __name__ == '__main__':
     unittest.main()
