@@ -1,32 +1,25 @@
 from __future__ import print_function
-from helper import unittest, PillowTestCase, hopper
 
+from helper import PillowTestCase, hopper, unittest
 from PIL import Image
 
 try:
-    import site
     import numpy
 except ImportError:
-    # Skip via setUp()
-    pass
+    numpy = None
+
 
 TEST_IMAGE_SIZE = (10, 10)
 
+
+@unittest.skipIf(numpy is None, "Numpy is not installed")
 class TestNumpy(PillowTestCase):
-
-    def setUp(self):
-        try:
-            import site
-            import numpy
-        except ImportError:
-            self.skipTest("ImportError")
-
     def test_numpy_to_image(self):
 
         def to_image(dtype, bands=1, boolean=0):
             if bands == 1:
                 if boolean:
-                    data = [0, 1] * 50
+                    data = [0, 255] * 50
                 else:
                     data = list(range(100))
                 a = numpy.array(data, dtype=dtype)
@@ -39,14 +32,14 @@ class TestNumpy(PillowTestCase):
                 a = numpy.array([[x]*bands for x in data], dtype=dtype)
                 a.shape = TEST_IMAGE_SIZE[0], TEST_IMAGE_SIZE[1], bands
                 i = Image.fromarray(a)
-                if list(i.split()[0].getdata()) != list(range(100)):
+                if list(i.getchannel(0).getdata()) != list(range(100)):
                     print("data mismatch for", dtype)
-            # print dtype, list(i.getdata())
+            # print(dtype, list(i.getdata()))
             return i
 
         # Check supported 1-bit integer formats
-        self.assertRaises(TypeError, lambda: to_image(numpy.bool))
-        self.assertRaises(TypeError, lambda: to_image(numpy.bool8))
+        self.assert_image(to_image(numpy.bool, 1, 1), '1', TEST_IMAGE_SIZE)
+        self.assert_image(to_image(numpy.bool8, 1, 1), '1', TEST_IMAGE_SIZE)
 
         # Check supported 8-bit integer formats
         self.assert_image(to_image(numpy.uint8), "L", TEST_IMAGE_SIZE)
@@ -73,12 +66,12 @@ class TestNumpy(PillowTestCase):
         self.assert_image(to_image(numpy.int32), "I", TEST_IMAGE_SIZE)
 
         # Check 64-bit integer formats
-        self.assertRaises(TypeError, lambda: to_image(numpy.uint64))
-        self.assertRaises(TypeError, lambda: to_image(numpy.int64))
+        self.assertRaises(TypeError, to_image, numpy.uint64)
+        self.assertRaises(TypeError, to_image, numpy.int64)
 
         # Check floating-point formats
         self.assert_image(to_image(numpy.float), "F", TEST_IMAGE_SIZE)
-        self.assertRaises(TypeError, lambda: to_image(numpy.float16))
+        self.assertRaises(TypeError, to_image, numpy.float16)
         self.assert_image(to_image(numpy.float32), "F", TEST_IMAGE_SIZE)
         self.assert_image(to_image(numpy.float64), "F", TEST_IMAGE_SIZE)
 
@@ -87,7 +80,7 @@ class TestNumpy(PillowTestCase):
         self.assert_image(to_image(numpy.uint8, 4), "RGBA", (10, 10))
 
     # based on an erring example at
-    # http://stackoverflow.com/questions/10854903/what-is-causing-dimension-dependent-attributeerror-in-pil-fromarray-function
+    # https://stackoverflow.com/questions/10854903/what-is-causing-dimension-dependent-attributeerror-in-pil-fromarray-function
     def test_3d_array(self):
         size = (5, TEST_IMAGE_SIZE[0], TEST_IMAGE_SIZE[1])
         a = numpy.ones(size, dtype=numpy.uint8)
@@ -100,6 +93,7 @@ class TestNumpy(PillowTestCase):
         self.assert_image(Image.fromarray(a[:, :, 1]), "L", TEST_IMAGE_SIZE)
 
     def _test_img_equals_nparray(self, img, np):
+        self.assertGreaterEqual(len(np.shape), 2)
         np_size = np.shape[1], np.shape[0]
         self.assertEqual(img.size, np_size)
         px = img.load()
@@ -113,20 +107,29 @@ class TestNumpy(PillowTestCase):
         self._test_img_equals_nparray(img, np_img)
         self.assertEqual(np_img.dtype, numpy.dtype('<u2'))
 
+    def test_1bit(self):
+        # Test that 1-bit arrays convert to numpy and back
+        # See: https://github.com/python-pillow/Pillow/issues/350
+        arr = numpy.array([[1, 0, 0, 1, 0], [0, 1, 0, 0, 0]], 'u1')
+        img = Image.fromarray(arr * 255).convert('1')
+        self.assertEqual(img.mode, '1')
+        arr_back = numpy.array(img)
+        # numpy 1.8 and earlier return this as a boolean. (trusty/precise)
+        if arr_back.dtype == numpy.bool:
+            arr_bool = numpy.array([[1, 0, 0, 1, 0], [0, 1, 0, 0, 0]], 'bool')
+            numpy.testing.assert_array_equal(arr_bool, arr_back)
+        else:
+            numpy.testing.assert_array_equal(arr, arr_back)
+
     def test_save_tiff_uint16(self):
-        '''
-        Open a single-channel uint16 greyscale image and verify that it can be saved without
-        losing precision.
-        '''
-        tmpfile = self.tempfile("temp.tif")
+        # Tests that we're getting the pixel value in the right byte order.
         pixel_value = 0x1234
-        filename = "Tests/images/uint16_1_4660.tif"
         a = numpy.array([pixel_value] * TEST_IMAGE_SIZE[0] * TEST_IMAGE_SIZE[1], dtype=numpy.uint16)
         a.shape = TEST_IMAGE_SIZE
-        Image.fromarray(a).save(tmpfile)
-        im_test = Image.open(tmpfile)
-        im_good = Image.open(filename)
-        self.assert_image_equal(im_good, im_test)
+        img = Image.fromarray(a)
+
+        img_px = img.load()
+        self.assertEqual(img_px[0, 0], pixel_value)
 
     def test_to_array(self):
 
@@ -179,8 +182,32 @@ class TestNumpy(PillowTestCase):
 
         self.assertEqual(len(im.getdata()), len(arr))
 
+    def test_zero_size(self):
+        # Shouldn't cause floating point exception
+        # See https://github.com/python-pillow/Pillow/issues/2259
+
+        im = Image.fromarray(numpy.empty((0, 0), dtype=numpy.uint8))
+
+        self.assertEqual(im.size, (0, 0))
+
+    def test_bool(self):
+        # https://github.com/python-pillow/Pillow/issues/2044
+        a = numpy.zeros((10, 2), dtype=numpy.bool)
+        a[0][0] = True
+
+        im2 = Image.fromarray(a)
+        self.assertEqual(im2.getdata()[0], 255)
+
+    def test_no_resource_warning_for_numpy_array(self):
+        # https://github.com/python-pillow/Pillow/issues/835
+        # Arrange
+        from numpy import array
+        test_file = 'Tests/images/hopper.png'
+        im = Image.open(test_file)
+
+        # Act/Assert
+        self.assert_warning(None, lambda: array(im))
+
 
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
